/*
* DVD-Audio Decoder plugin
* Copyright (c) 2009 Maxim V.Anisiutkin <maxim.anisiutkin@gmail.com>
*
* DVD-Audio Decoder is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* DVD-Audio Decoder is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with FFmpeg; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef _DVDA_DISC_CACHE
#define _DVDA_DISC_CACHE

#include <vector>
#include <windows.h>
#include "audio_track.h"
#include "dvda_metabase.h"
#include "dvda_zone.h"
#include "foo_version.h"

class dvda_zone_key_t {
public:
	size_t   size;
	uint64_t timestamp;
	bool operator<(dvda_zone_key_t& dvda_zone_key) const {
		return true;
	}
};

class dvda_disc_t {
public:
	t_filestats        dvda_stats;
	dvda_filesystem_t* dvda_filesystem;
	dvda_zone_t*       dvda_zone;
	track_list_t*      track_list;
	dvda_metabase_t*   dvda_metabase;
};

class dvda_disc_cache_t {
	CRITICAL_SECTION lock;
	std::vector<dvda_disc_t> dvda_discs;
public:
	volatile LONG references;
	DWORD thread_id;

	dvda_disc_cache_t() {
		InitializeCriticalSection(&lock);
		references = 0;
		thread_id = 0;
	}

	~dvda_disc_cache_t() {
		DeleteCriticalSection(&lock);
	}

	void add_ref() {
		InterlockedIncrement(&references);
	}

	void release() {
		InterlockedDecrement(&references);
		if (references <= 0)
			delayed_flush();
	}

	void add(dvda_disc_t dvda_disc) {
		EnterCriticalSection(&lock);
		dvda_discs.push_back(dvda_disc);
		LeaveCriticalSection(&lock);
	}

	bool cached(t_filestats dvda_stats) {
		bool in_cache;
		EnterCriticalSection(&lock);
		in_cache = false;
		for (std::vector<dvda_disc_t>::size_type i = 0; i < dvda_discs.size(); i++) {
			if (dvda_discs[i].dvda_stats == dvda_stats) {
				in_cache = true;
				break;
			}
		}
		LeaveCriticalSection(&lock);
		return in_cache;
	}

	bool cached(t_filestats dvda_stats, dvda_filesystem_t** p_dvda_filesystem, dvda_zone_t** p_dvda_zone, track_list_t** p_track_list, dvda_metabase_t** p_dvda_metabase) {
		bool in_cache;
		EnterCriticalSection(&lock);
		in_cache = false;
		for (std::vector<dvda_disc_t>::size_type i = 0; i < dvda_discs.size(); i++) {
			if (dvda_discs[i].dvda_stats == dvda_stats) {
				*p_dvda_filesystem = dvda_discs[i].dvda_filesystem;
				*p_dvda_zone = dvda_discs[i].dvda_zone;
				*p_track_list = dvda_discs[i].track_list;
				*p_dvda_metabase = dvda_discs[i].dvda_metabase;
				in_cache = true;
				break;
			}
		}
		LeaveCriticalSection(&lock);
		return in_cache;
	}

	dvda_disc_t& operator[](std::vector<dvda_disc_t>::size_type i) {
		EnterCriticalSection(&lock);
		dvda_disc_t& dvda_disc = dvda_discs[i];
		LeaveCriticalSection(&lock);
		return dvda_disc;
	}

	void flush() {
		EnterCriticalSection(&lock);
		if (dvda_discs.size() > 0) {
			console_printf("Release cached DVD-Audio discs");
			for (std::vector<dvda_disc_t>::size_type i = 0; i < dvda_discs.size(); i++) {
				if (dvda_discs[i].dvda_zone) {
					dvda_discs[i].dvda_zone->close();
					delete dvda_discs[i].dvda_zone;
				}
				if (dvda_discs[i].dvda_filesystem) {
					dvda_discs[i].dvda_filesystem->dismount();
					delete dvda_discs[i].dvda_filesystem;
				}
				if (dvda_discs[i].track_list) {
					dvda_discs[i].track_list->clear();
					delete dvda_discs[i].track_list;
				}
				if (dvda_discs[i].dvda_metabase)
					delete dvda_discs[i].dvda_metabase;
			}
			dvda_discs.clear();
		}
		LeaveCriticalSection(&lock);
	}

	void delayed_flush();
};

#endif
