/*
* DVD-Audio Decoder plugin
* Copyright (c) 2009 Maxim V.Anisiutkin <maxim.anisiutkin@gmail.com>
*
* DVD-Audio Decoder is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* DVD-Audio Decoder is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with FFmpeg; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef _DVDA_FILE
#define _DVDA_FILE

#include <stdint.h>

//#define _RTL_DVDA_FILE

#ifdef _RTL_DVDA_FILE

#include <string>
#include <sys/stat.h>

class dvda_file_t {
	std::string dvda_zone;
	FILE* stream;
public:
	dvda_file_t() {
		dvda_zone = ".";
		stream = NULL;
	}
	void set_path(const char* zone_path) {
		dvda_zone = zone_path;
		stream = NULL;
	}
	bool open(const char* file_name)  {
		std::string::size_type pos = dvda_zone.find("://");
		if (pos != std::string::npos)
			pos += 3;
		else
			pos = 0;
		stream = fopen((dvda_zone.substr(pos) + "\\" + file_name).c_str(), "rb");
		if (stream == NULL)
			return false;
		return (stream != NULL);
	}
	bool is_open() {
		return stream != NULL;
	}
	bool close() {
		if (stream != NULL) {
			int err = fclose(stream);
			stream = NULL;
			return err == 0;
		}
		return false;
	}
	int read(void* buffer, int count) {
		return fread(buffer, 1, count, stream);
	}
	bool seek(int64_t offset, int origin = SEEK_SET) {
		return _fseeki64(stream, offset, SEEK_SET) == 0;
	}
	int64_t size(const char* file_name) {
		std::string::size_type pos = dvda_zone.find("://");
		if (pos != std::string::npos)
			pos += 3;
		else
			pos = 0;
		struct _stati64 st;
		if (_stati64((dvda_zone.substr(pos) + "\\" + file_name).c_str(), &st) == 0)
			return st.st_size;
		return -1;
	}
};

#else

#include <sys/stat.h>
#include "foo_version.h"

#ifdef FOO_VERSION_10

class dvda_file_t {
	abort_callback_impl abort;
	service_ptr_t<file> stream;
public:
	bool open(const char* file_path) {
		if (filesystem::g_exists(file_path, abort)) {
			filesystem::g_open_read(stream, file_path, abort);
			return true;
		}
		return false;
	}
	bool is_open() {
		return stream.is_valid();
	}
	bool close() {
		if (stream.is_valid()) {
			stream.release();
			return true;
		}
		return false;
	}
	int read(void* buffer, int count) {
		return (int)stream->read(buffer, count, abort);
	}
	bool seek(int64_t offset, int origin = SEEK_SET) {
		stream->seek(offset, abort);
		return true;
	}
	int64_t get_size(const char* file_path) {
		bool file_exists = false;
		try {
			file_exists = filesystem::g_exists(file_path, abort);
		}
		catch (...) {
		}
		if (file_exists) {
			t_filestats stats;
			bool is_writable;
			filesystem::g_get_stats(file_path, stats, is_writable, abort);
			return stats.m_size;
		}
		return -1;
	}
};

#endif

#ifdef FOO_VERSION_08X

class dvda_file_t {
	reader* stream;
public:
	dvda_file_t() {
		stream = 0;
	}
	bool open(const char* file_path) {
		if (file::g_exists(file_path)) {
			stream = file::g_open_read(file_path);
			return stream != 0;
		}
		return false;
	}
	bool is_open() {
		return stream != 0;
	}
	bool close() {
		if (stream) {
			//stream->reader_release();
			stream = 0;
			return true;
		}
		return false;
	}
	int read(void* buffer, int count) {
		return (int)stream->read(buffer, count);
	}
	bool seek(int64_t offset, int origin = SEEK_SET) {
		return stream->seek(offset);
	}
	int64_t get_size(const char* file_path) {
		int file_exists = file::g_exists(file_path);
		int64_t size = -1;
		if ((file_exists & file::FILE_EXISTS) == file::FILE_EXISTS) {
			if (stream)
				size = stream->get_length();
		}
		return size;
	}
};

#endif

#endif

#endif
