/*
* DVD-Audio Decoder plugin
* Copyright (c) 2009 Maxim V.Anisiutkin <maxim.anisiutkin@gmail.com>
*
* DVD-Audio Decoder is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* DVD-Audio Decoder is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with FFmpeg; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef _FOO_SETUP
#define _FOO_SETUP

static const GUID g_guid_cfg_no_downmixes = {0x4268c62d, 0x48a2, 0x4438, {0xb7, 0x9, 0xea, 0x62, 0x8, 0x8e, 0xaf, 0x3d}};
static cfg_uint g_cfg_no_downmixes(g_guid_cfg_no_downmixes, BST_UNCHECKED);

static const GUID g_guid_cfg_no_short_tracks = {0x4f444d0d, 0xb8fd, 0x44ba, {0xa9, 0x17, 0xa7, 0x5a, 0x25, 0x7, 0x1e, 0x33}};
static cfg_uint g_cfg_no_short_tracks(g_guid_cfg_no_short_tracks, BST_UNCHECKED);

class CDVDAPreferences : public CDialogImpl<CDVDAPreferences>, public preferences_page_instance {
public:
	static bool get_cfg_no_downmixes() {
		return g_cfg_no_downmixes == BST_CHECKED;
	}

	static bool get_cfg_no_short_tracks() {
		return g_cfg_no_short_tracks == BST_CHECKED;
	}

	CDVDAPreferences(preferences_page_callback::ptr callback) : m_callback(callback) {
	}

	enum {IDD = IDD_DVDAPREFERENCES};

	t_uint32 get_state() {
		t_uint32 state = preferences_state::resettable;
		if (HasChanged())
			state |= preferences_state::changed;
		return state;
	}

	void apply() {
		g_cfg_no_downmixes = SendDlgItemMessage(IDC_NO_DOWNMIXES, BM_GETCHECK, 0, 0);
		g_cfg_no_short_tracks = SendDlgItemMessage(IDC_NO_SHORT_TRACKS, BM_GETCHECK, 0, 0);
		OnChanged();
	}

	void reset() {
		SendDlgItemMessage(IDC_NO_DOWNMIXES, BM_SETCHECK, BST_UNCHECKED, 0);
		SendDlgItemMessage(IDC_NO_SHORT_TRACKS, BM_SETCHECK, BST_UNCHECKED, 0);
		OnChanged();
	}

	BEGIN_MSG_MAP(CDVDAPreferences)
		MSG_WM_INITDIALOG(OnInitDialog)
		COMMAND_HANDLER_EX(IDC_NO_DOWNMIXES, BN_CLICKED, OnPropertyChange)
		COMMAND_HANDLER_EX(IDC_NO_SHORT_TRACKS, BN_CLICKED, OnPropertyChange)
	END_MSG_MAP()

private:
	BOOL OnInitDialog(CWindow, LPARAM) {
		SendDlgItemMessage(IDC_NO_DOWNMIXES, BM_SETCHECK, g_cfg_no_downmixes, 0);
		SendDlgItemMessage(IDC_NO_SHORT_TRACKS, BM_SETCHECK, g_cfg_no_short_tracks, 0);
		return TRUE;
	}

	void OnPropertyChange(UINT, int, CWindow) {
		OnChanged();
	}

	bool HasChanged() {
		return
			SendDlgItemMessage(IDC_NO_DOWNMIXES, BM_GETCHECK, 0, 0) != g_cfg_no_downmixes ||
			SendDlgItemMessage(IDC_NO_SHORT_TRACKS, BM_GETCHECK, 0, 0) != g_cfg_no_short_tracks;
	}

	void OnChanged() {
		m_callback->on_state_changed();
	}

	const preferences_page_callback::ptr m_callback;
};

class preferences_page_dvda_t : public preferences_page_impl<CDVDAPreferences> {
public:
	const char* get_name() {
		return "DVD-Audio";
	}
	GUID get_guid() {
		static const GUID guid = {0xe29b38e5, 0x5ecd, 0x4c85, {0x95, 0x30, 0x7f, 0x61, 0x39, 0x10, 0x20, 0xd}};
		return guid;
	}
	GUID get_parent_guid() {
		return guid_tools;
	}
};

static preferences_page_factory_t<preferences_page_dvda_t> g_preferences_page_dvda_factory;

#endif