/*
* DVD-Audio Decoder plugin
* Copyright (c) 2009 Maxim V.Anisiutkin <maxim.anisiutkin@gmail.com>
*
* DVD-Audio Decoder is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* DVD-Audio Decoder is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with FFmpeg; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef _FOO_VERSI0N
#define _FOO_VERSI0N

#ifdef FOO_VERSION_08X
#pragma warning(disable:4996)
#endif

#include <foobar2000.h>

#ifdef FOO_VERSION_10

using namespace pfc;

typedef array_t<t_uint8> byte_array_t;

typedef map_t<t_size, file_info_impl> file_info_array_t;

typedef service_ptr_t<file> media_file_t;

typedef audio_chunk audio_chunk_t;

inline void console_printfv(const char* fmt, va_list vl) {
	console::printfv(fmt, vl);
}

inline void console_printf(const char* fmt, ...) {
	va_list vl;
	va_start(vl, fmt);
	console_printfv(fmt, vl);
	va_end(vl);
}

using namespace stringcvt;

#endif

#ifdef FOO_VERSION_08X

#include <stdint.h>
#include <exception>

typedef int8_t   t_int8;
typedef int16_t  t_int16;
typedef int32_t  t_int32;
typedef int64_t  t_int64;
typedef uint8_t  t_uint8;
typedef uint16_t t_uint16;
typedef uint32_t t_uint32;
typedef uint64_t t_uint64;
typedef uint32_t t_size;
typedef uint64_t t_filesize;
typedef uint64_t t_filetimestamp;

typedef mem_block_t<t_uint8> byte_array_t;

#define PFC_DECLARE_EXCEPTION(NAME, BASECLASS, DEFAULTMSG)	\
class NAME : public BASECLASS {	\
public:	\
	static const char* g_what() {return DEFAULTMSG;}	\
	NAME() : BASECLASS(DEFAULTMSG, 0) {}	\
	NAME(const char* p_msg) : BASECLASS(p_msg) {}	\
	NAME(const char* p_msg, int) : BASECLASS(p_msg, 0) {}	\
	NAME(const NAME& p_source) : BASECLASS(p_source) {}	\
};

PFC_DECLARE_EXCEPTION(exception_io,                       std::exception,      "I/O error");
PFC_DECLARE_EXCEPTION(exception_io_not_found,             exception_io,        "Object not found");
PFC_DECLARE_EXCEPTION(exception_io_denied,                exception_io,        "Access denied");
PFC_DECLARE_EXCEPTION(exception_io_data,                  exception_io,        "Unsupported format or corrupted file");
PFC_DECLARE_EXCEPTION(exception_io_data_truncation,       exception_io_data,   "Unsupported format or corrupted file");
PFC_DECLARE_EXCEPTION(exception_io_unsupported_format,    exception_io_data,   "Unsupported file format");
PFC_DECLARE_EXCEPTION(exception_io_object_is_remote,      exception_io,        "This operation is not supported on remote objects");
PFC_DECLARE_EXCEPTION(exception_io_sharing_violation,     exception_io,        "Sharing violation");
PFC_DECLARE_EXCEPTION(exception_io_device_full,           exception_io,        "Device full");
PFC_DECLARE_EXCEPTION(exception_io_seek_out_of_range,     exception_io,        "Seek offset out of range");
PFC_DECLARE_EXCEPTION(exception_io_object_not_seekable,   exception_io,        "Object is not seekable");
PFC_DECLARE_EXCEPTION(exception_io_no_length,             exception_io,        "Length of object is unknown");
PFC_DECLARE_EXCEPTION(exception_io_no_handler_for_path,   exception_io,        "Invalid path");
PFC_DECLARE_EXCEPTION(exception_io_already_exists,        exception_io,        "Object already exists");
PFC_DECLARE_EXCEPTION(exception_io_no_data,               exception_io,        "The process receiving or sending data has terminated");
PFC_DECLARE_EXCEPTION(exception_io_network_not_reachable, exception_io,        "Network not reachable");
PFC_DECLARE_EXCEPTION(exception_io_write_protected,       exception_io_denied, "The media is write protected");
PFC_DECLARE_EXCEPTION(exception_io_file_corrupted,        exception_io,        "The file is corrupted");
PFC_DECLARE_EXCEPTION(exception_io_disk_change,           exception_io,        "Disc not available");
PFC_DECLARE_EXCEPTION(exception_io_directory_not_empty,   exception_io,        "Directory not empty");

struct t_filestats {
	t_filesize m_size;
	t_filetimestamp m_timestamp;
	inline bool operator==(const t_filestats & param) const {return m_size == param.m_size && m_timestamp == param.m_timestamp;}
	inline bool operator!=(const t_filestats & param) const {return m_size != param.m_size || m_timestamp != param.m_timestamp;}
};

class abort_callback {
};

class file_reader_t {
	reader* m_reader;
public:
	file_reader_t() {
		m_reader = NULL;
	}
	void set(reader* p_reader) {
		m_reader = p_reader;
	}
	bool is_empty() {
		return m_reader == NULL;
	}
	t_filestats get_stats(abort_callback& p_abort) {
		t_filestats filestats;
		filestats.m_size = 0;
		filestats.m_timestamp = 0;
		if (m_reader) {
			filestats.m_size = m_reader->get_length();
			filestats.m_timestamp = m_reader->get_modification_time();
		}
		return filestats;
	}
	void reopen(abort_callback& p_abort) {
		if (m_reader)
			m_reader->seek(0);
	}
	t_size read(void* p_buffer, t_size p_bytes, abort_callback& p_abort) {
		return m_reader->read(p_buffer, p_bytes);
	}
	t_filesize get_size(abort_callback& p_abort) {
		return m_reader->get_length();
	}
	void ensure_seekable() {
		if (!m_reader->can_seek())
			throw exception_io_object_not_seekable();
	}
	void seek(t_filesize p_position, abort_callback& p_abort) {
		if (!m_reader->seek(p_position))
			throw exception_io_seek_out_of_range();
	}
	bool can_seek() {
		return m_reader->can_seek();
	}
	void on_idle(abort_callback& p_abort) {
	}
};

typedef file_reader_t* media_file_t;

enum t_input_open_reason {
	input_open_info_read,
	input_open_decode,
	input_open_info_write
};

inline void input_open_file_helper(media_file_t& p_file, const char* p_path, t_input_open_reason p_reason, abort_callback & p_abort) {
	if (p_file->is_empty()) {
		switch (p_reason) {
		case input_open_info_read:
		case input_open_decode:
			p_file->set(file::g_open(p_path, reader::MODE_READ));
			break;
		case input_open_info_write:
			p_file->set(file::g_open(p_path, reader::MODE_WRITE_EXISTING));
			break;
		}
	} else {
		p_file->reopen(p_abort);
	}
}

class audio_chunk_t {
	audio_chunk* m_chunk;
public:
	enum {
		channel_front_left         = 1<<0,
		channel_front_right        = 1<<1,
		channel_front_center       = 1<<2,
		channel_lfe                = 1<<3,
		channel_back_left          = 1<<4,
		channel_back_right         = 1<<5,
		channel_front_center_left  = 1<<6,
		channel_front_center_right = 1<<7,
		channel_back_center        = 1<<8,
		channel_side_left          = 1<<9,
		channel_side_right         = 1<<10,
		channel_top_center         = 1<<11,
		channel_top_front_left     = 1<<12,
		channel_top_front_center   = 1<<13,
		channel_top_front_right    = 1<<14,
		channel_top_back_left      = 1<<15,
		channel_top_back_center    = 1<<16,
		channel_top_back_right     = 1<<17,

		channel_config_mono        = channel_front_center,
		channel_config_stereo      = channel_front_left | channel_front_right,
		channel_config_5point1     = channel_front_left | channel_front_right | channel_back_left | channel_back_right | channel_front_center | channel_lfe,

		defined_channel_count      = 18
	};
	audio_chunk_t() {
		m_chunk = NULL;
	}
	void set(audio_chunk* p_chunk) {
		m_chunk = p_chunk;
	}
	audio_sample* get_data() const {
		return m_chunk->get_data();
	}
	unsigned get_channels() const {
		return m_chunk->get_channels();
	}
	t_size get_sample_count() const {
		return m_chunk->get_sample_count();
	}
	unsigned get_sample_rate() const {
		return m_chunk->get_srate();
	}
	unsigned get_srate() const {
		return m_chunk->get_srate();
	}
	static unsigned g_guess_channel_config(unsigned count);
	static unsigned g_channel_config_from_wfx(DWORD p_wfx);
	void set_data_fixedpoint(const void* ptr, t_size bytes, unsigned srate, unsigned nch, unsigned bps, unsigned channel_config);
};

inline void console_printfv(const char* fmt, va_list vl) {
	char str[1024];
	vsprintf_s(str, sizeof(str), fmt, vl);
	console::info(str);
}

inline void console_printf(const char* fmt, ...) {
	va_list vl;
	va_start(vl, fmt);
	console_printfv(fmt, vl);
	va_end(vl);
}

class string_formatter : public string8_fastalloc {
public:
	inline string_formatter& operator<<(string_formatter& p_source) {
		*this += p_source;
		return *this;
	}
	inline string_formatter& operator<<(const char* p_source) {
		add_string(p_source);
		return *this;
	}
	inline string_base& operator<<(int p_val) {
		add_int(p_val);
		return *this;
	}
	inline string_base& operator<<(t_int32 p_val) {
		add_int(p_val);
		return *this;
	}
	inline string_base& operator<<(t_uint32 p_val) {
		add_uint(p_val);
		return *this;
	}
	inline string_base& operator<<(t_int64 p_val) {
		add_int(p_val);
		return *this;
	}
	inline string_base& operator<<(t_uint64 p_val) {
		add_uint(p_val);
		return *this;
	}
	inline string_base& operator<<(double p_val) {
		add_float(p_val, 10);
		return *this;
	}
};

class string_directory {
	string8 m_data;
public:
	string_directory(const char* p_path) {
		t_size ptr = string8::g_scan_filename(p_path);
		if (ptr > 0)
			m_data.set_string(p_path, ptr - 1);
	}

	inline operator const char*() const {
		return m_data;
	}
};

t_size string_find_first(const char* p_string, const char* p_tofind, t_size p_start = 0);

namespace audio_math {
	inline double samples_to_time(t_uint64 p_samples,t_uint32 p_sample_rate) {
		return (double) p_samples / (double) p_sample_rate;
	}
}

class dsp_impl_base {
public:
	virtual bool need_track_change_mark() {
		return true;
	}
};

template<typename T>
class initquit_factory_t : public service_factory_single_t<initquit, T> {};

#endif

#endif
